@@ -6,6 +6,7 @@
 import datetime
 
 from django.db import models
+from django.db import transaction
 from django.conf import settings
 from django.contrib.contenttypes import generic
 from django.contrib.contenttypes import models as ct_models
@@ -24,8 +25,13 @@ transition = base.transition
 
 
 class StateSelect(widgets.Select):
+    """Custom 'select' widget to handle state retrieval."""
 
     def render(self, name, value, attrs=None, choices=()):
+        """Handle a few expected values for rendering the current choice.
+
+        Extracts the state name from StateWrapper and State object.
+        """
         if isinstance(value, base.StateWrapper):
             state_name = value.state.name
         elif isinstance(value, base.State):
@@ -154,20 +160,25 @@ class StateField(models.Field):
     def formfield(self, form_class=fields.ChoiceField, widget=StateSelect, **kwargs):
         return super(StateField, self).formfield(form_class, widget=widget, **kwargs)
 
+    def south_field_triple(self):
+        """Return a suitable description of this field for South."""
+        from south.modelsinspector import introspector
+        args, kwargs = introspector(self)
 
-    try:
-        import south
-    except ImportError:
-        pass
-    else:
-        def south_field_triple(self):
-            """Return a suitable description of this field for South."""
-            workflow_class = type(self.workflow)
-            workflow = ("__import__('%(mod)s', globals(), locals(), '%(cls)s').%(cls)s"
-                    % {'mod': workflow_class.__module__, 'cls': workflow_class.__name__})
-            from south.modelsinspector import introspector
-            args, kwargs = introspector(self)
-            return ('django_xworkflows.models.StateField', [workflow] + args, kwargs)
+        state_def = tuple((st.name, st.title) for st in self.workflow.states)
+        initial_state_def = self.workflow.initial_state.name
+
+        workflow = (
+            "__import__('xworkflows', globals(), locals()).base.WorkflowMeta("
+            "'%(class_name)s', (), "
+            "{'states': %(states)r, 'initial_state': %(initial_state)r})" % {
+                'class_name': self.workflow.__class__.__name__,
+                'states': state_def,
+                'initial_state': initial_state_def,
+            })
+        kwargs['workflow'] = workflow
+
+        return ('django_xworkflows.models.StateField', args, kwargs)
 
 
 class WorkflowEnabledMeta(base.WorkflowEnabledMeta, models.base.ModelBase):
@@ -175,7 +186,11 @@ class WorkflowEnabledMeta(base.WorkflowEnabledMeta, models.base.ModelBase):
 
     @classmethod
     def _find_workflows(mcs, attrs):
-        """Find workflow definition(s) in a WorkflowEnabled definition."""
+        """Find workflow definition(s) in a WorkflowEnabled definition.
+
+        This method overrides the default behavior from xworkflows in order to
+        use our custom StateField objects.
+        """
         workflows = {}
         for k, v in attrs.iteritems():
             if isinstance(v, StateField):
@@ -190,6 +205,9 @@ class WorkflowEnabledMeta(base.WorkflowEnabledMeta, models.base.ModelBase):
         """
         attrs[field_name] = state_field
 
+        implems = base.ImplementationList(field_name, state_field.workflow)
+        implems.transform(attrs)
+
 
 class WorkflowEnabled(base.BaseWorkflowEnabled):
     """Base class for all django models wishing to use a Workflow."""
@@ -248,12 +266,23 @@ def get_default_log_model():
         return ''
 
 
+class TransactionalImplementationWrapper(base.ImplementationWrapper):
+    """Customize the base ImplementationWrapper to run into a db transaction."""
+
+    def __call__(self, *args, **kwargs):
+        with transaction.commit_on_success():
+            return super(TransactionalImplementationWrapper, self).__call__(*args, **kwargs)
+
+
 class Workflow(base.Workflow):
     """Extended workflow that handles object saving and logging to the database."""
 
     #: Save log to this django model
     log_model = get_default_log_model()
 
+    #: Run all transitions withi a transaction.
+    implementation_class = TransactionalImplementationWrapper
+
     def __init__(self, log_model=None, *args, **kwargs):
         super(Workflow, self).__init__(*args, **kwargs)
 
